# Stubs for xlrd.xldate (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

epoch_1904: Any
epoch_1900: Any
epoch_1900_minus_1: Any

class XLDateError(ValueError): ...
class XLDateNegative(XLDateError): ...
class XLDateAmbiguous(XLDateError): ...
class XLDateTooLarge(XLDateError): ...
class XLDateBadDatemode(XLDateError): ...
class XLDateBadTuple(XLDateError): ...

def xldate_as_tuple(xldate: Any, datemode: Any): ...
def xldate_as_datetime(xldate: Any, datemode: Any): ...
def xldate_from_date_tuple(date_tuple: Any, datemode: Any): ...
def xldate_from_time_tuple(time_tuple: Any): ...
def xldate_from_datetime_tuple(datetime_tuple: Any, datemode: Any): ...
