# Stubs for xlwt.ExcelFormulaLexer (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .antlr import TokenStream
from typing import Any

int_const_pattern: str
flt_const_pattern: str
str_const_pattern: str
ref2d_r1c1_pattern: str
ref2d_pattern: str
true_pattern: str
false_pattern: str
if_pattern: str
choose_pattern: str
name_pattern: str
quotename_pattern: str
ne_pattern: str
ge_pattern: str
le_pattern: str
pattern_type_tuples: Any
single_char_lookup: Any

class Lexer(TokenStream):
    def __init__(self, text: Any) -> None: ...
    def isEOF(self): ...
    def curr_ch(self): ...
    def next_ch(self, n: int = ...) -> None: ...
    def is_whitespace(self): ...
    def match_pattern(self): ...
    def nextToken(self): ...
