# Stubs for xlwt.Style (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .Formatting import Font
from typing import Any, Optional

FIRST_USER_DEFINED_NUM_FORMAT_IDX: int

class XFStyle:
    num_format_str: str = ...
    font: Any = ...
    alignment: Any = ...
    borders: Any = ...
    pattern: Any = ...
    protection: Any = ...
    def __init__(self) -> None: ...

default_style: Any

class StyleCollection:
    style_compression: Any = ...
    stats: Any = ...
    default_style: Any = ...
    def __init__(self, style_compression: int = ...) -> None: ...
    def add(self, style: Any): ...
    def add_font(self, font: Any): ...
    def get_biff_data(self): ...

class EasyXFException(Exception): ...
class EasyXFCallerError(EasyXFException): ...
class EasyXFAuthorError(EasyXFException): ...

class IntULim:
    limit: Any = ...
    def __init__(self, limit: Any) -> None: ...
    def __call__(self, astring: Any): ...

bool_map: Any
border_line_map: Any
charset_map: Any
colour_map: Any

def add_palette_colour(colour_str: Any, colour_index: Any) -> None: ...

excel_default_palette_b8: Any
pattern_map: Any

def any_str_func(s: Any): ...
def colour_index_func(s: Any, maxval: int = ...): ...
colour_index_func_7 = colour_index_func

def colour_index_func_15(s: Any): ...
def rotation_func(s: Any): ...

xf_dict: Any

def easyxf(strg_to_parse: str = ..., num_format_str: Optional[Any] = ..., field_sep: str = ..., line_sep: str = ..., intro_sep: str = ..., esc_char: str = ..., debug: bool = ...) -> XFStyle: ...
def easyfont(strg_to_parse: str = ..., field_sep: str = ..., esc_char: str = ..., debug: bool = ...) -> Font: ...
