# Stubs for xlwt.ExcelFormula (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Formula:
    def __init__(self, s: Any) -> None: ...
    def get_references(self): ...
    def patch_references(self, patches: Any) -> None: ...
    def text(self): ...
    def rpn(self): ...
