# Stubs for xlwt.ExcelMagic (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

MAX_ROW: int
MAX_COL: int
biff_records: Any
all_funcs_by_name: Any
ptgExp: int
ptgTbl: int
ptgAdd: int
ptgSub: int
ptgMul: int
ptgDiv: int
ptgPower: int
ptgConcat: int
ptgLT: int
ptgLE: int
ptgEQ: int
ptgGE: int
ptgGT: int
ptgNE: int
ptgIsect: int
ptgUnion: int
ptgRange: int
ptgUplus: int
ptgUminus: int
ptgPercent: int
ptgParen: int
ptgMissArg: int
ptgStr: int
ptgExtend: int
ptgAttr: int
ptgSheet: int
ptgEndSheet: int
ptgErr: int
ptgBool: int
ptgInt: int
ptgNum: int
ptgArrayR: int
ptgFuncR: int
ptgFuncVarR: int
ptgNameR: int
ptgRefR: int
ptgAreaR: int
ptgMemAreaR: int
ptgMemErrR: int
ptgMemNoMemR: int
ptgMemFuncR: int
ptgRefErrR: int
ptgAreaErrR: int
ptgRefNR: int
ptgAreaNR: int
ptgMemAreaNR: int
ptgMemNoMemNR: int
ptgNameXR: int
ptgRef3dR: int
ptgArea3dR: int
ptgRefErr3dR: int
ptgAreaErr3dR: int
ptgArrayV: int
ptgFuncV: int
ptgFuncVarV: int
ptgNameV: int
ptgRefV: int
ptgAreaV: int
ptgMemAreaV: int
ptgMemErrV: int
ptgMemNoMemV: int
ptgMemFuncV: int
ptgRefErrV: int
ptgAreaErrV: int
ptgRefNV: int
ptgAreaNV: int
ptgMemAreaNV: int
ptgMemNoMemNV: int
ptgFuncCEV: int
ptgNameXV: int
ptgRef3dV: int
ptgArea3dV: int
ptgRefErr3dV: int
ptgAreaErr3dV: int
ptgArrayA: int
ptgFuncA: int
ptgFuncVarA: int
ptgNameA: int
ptgRefA: int
ptgAreaA: int
ptgMemAreaA: int
ptgMemErrA: int
ptgMemNoMemA: int
ptgMemFuncA: int
ptgRefErrA: int
ptgAreaErrA: int
ptgRefNA: int
ptgAreaNA: int
ptgMemAreaNA: int
ptgMemNoMemNA: int
ptgFuncCEA: int
ptgNameXA: int
ptgRef3dA: int
ptgArea3dA: int
ptgRefErr3dA: int
ptgAreaErr3dA: int
PtgNames: Any
error_msg_by_code: Any
