# Stubs for xlrd.timemachine (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

python_version: Any
BYTES_LITERAL: Any
UNICODE_LITERAL: Any
BYTES_ORD: Any

def fprintf(f: Any, fmt: Any, *vargs: Any) -> None: ...

EXCEL_TEXT_TYPES: Any
REPR = ascii
xrange = range
unicode: Any
ensure_unicode: Any
unichr = chr
