# Stubs for xlwt.BIFFRecords (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

class SharedStringTable:
    encoding: Any = ...
    def __init__(self, encoding: Any) -> None: ...
    def add_str(self, s: Any): ...
    def add_rt(self, rt: Any): ...
    def del_str(self, idx: Any) -> None: ...
    def str_index(self, s: Any): ...
    def rt_index(self, rt: Any): ...
    def get_biff_record(self): ...

class BiffRecord:
    def get_rec_header(self): ...
    def get(self): ...

class Biff8BOFRecord(BiffRecord):
    BOOK_GLOBAL: int = ...
    VB_MODULE: int = ...
    WORKSHEET: int = ...
    CHART: int = ...
    MACROSHEET: int = ...
    WORKSPACE: int = ...
    def __init__(self, rec_type: Any) -> None: ...

class InteraceHdrRecord(BiffRecord):
    def __init__(self) -> None: ...

class InteraceEndRecord(BiffRecord):
    def __init__(self) -> None: ...

class MMSRecord(BiffRecord):
    def __init__(self) -> None: ...

class WriteAccessRecord(BiffRecord):
    def __init__(self, owner: Any) -> None: ...

class DSFRecord(BiffRecord):
    def __init__(self) -> None: ...

class TabIDRecord(BiffRecord):
    def __init__(self, sheetcount: Any) -> None: ...

class FnGroupCountRecord(BiffRecord):
    def __init__(self) -> None: ...

class WindowProtectRecord(BiffRecord):
    def __init__(self, wndprotect: Any) -> None: ...

class ObjectProtectRecord(BiffRecord):
    def __init__(self, objprotect: Any) -> None: ...

class ScenProtectRecord(BiffRecord):
    def __init__(self, scenprotect: Any) -> None: ...

class ProtectRecord(BiffRecord):
    def __init__(self, protect: Any) -> None: ...

class PasswordRecord(BiffRecord):
    def passwd_hash(self, plaintext: Any): ...
    def __init__(self, passwd: str = ...) -> None: ...

class Prot4RevRecord(BiffRecord):
    def __init__(self) -> None: ...

class Prot4RevPassRecord(BiffRecord):
    def __init__(self) -> None: ...

class BackupRecord(BiffRecord):
    def __init__(self, backup: Any) -> None: ...

class HideObjRecord(BiffRecord):
    def __init__(self) -> None: ...

class RefreshAllRecord(BiffRecord):
    def __init__(self) -> None: ...

class BookBoolRecord(BiffRecord):
    def __init__(self) -> None: ...

class CountryRecord(BiffRecord):
    def __init__(self, ui_id: Any, sys_settings_id: Any) -> None: ...

class UseSelfsRecord(BiffRecord):
    def __init__(self) -> None: ...

class EOFRecord(BiffRecord):
    def __init__(self) -> None: ...

class DateModeRecord(BiffRecord):
    def __init__(self, from1904: Any) -> None: ...

class PrecisionRecord(BiffRecord):
    def __init__(self, use_real_values: Any) -> None: ...

class CodepageBiff8Record(BiffRecord):
    UTF_16: int = ...
    def __init__(self) -> None: ...

class Window1Record(BiffRecord):
    def __init__(self, hpos_twips: Any, vpos_twips: Any, width_twips: Any, height_twips: Any, flags: Any, active_sheet: Any, first_tab_index: Any, selected_tabs: Any, tab_width: Any) -> None: ...

class FontRecord(BiffRecord):
    def __init__(self, height: Any, options: Any, colour_index: Any, weight: Any, escapement: Any, underline: Any, family: Any, charset: Any, name: Any) -> None: ...

class NumberFormatRecord(BiffRecord):
    def __init__(self, idx: Any, fmtstr: Any) -> None: ...

class XFRecord(BiffRecord):
    def __init__(self, xf: Any, xftype: str = ...) -> None: ...

class StyleRecord(BiffRecord):
    def __init__(self) -> None: ...

class PaletteRecord(BiffRecord):
    def __init__(self, custom_palette: Any) -> None: ...

class BoundSheetRecord(BiffRecord):
    def __init__(self, stream_pos: Any, visibility: Any, sheetname: Any, encoding: str = ...) -> None: ...

class ContinueRecord(BiffRecord): ...
class SSTRecord(BiffRecord): ...

class ExtSSTRecord(BiffRecord):
    def __init__(self, sst_stream_pos: Any, str_placement: Any, portions_len: Any) -> None: ...

class DimensionsRecord(BiffRecord):
    def __init__(self, first_used_row: Any, last_used_row: Any, first_used_col: Any, last_used_col: Any) -> None: ...

class Window2Record(BiffRecord):
    def __init__(self, options: Any, first_visible_row: Any, first_visible_col: Any, grid_colour: Any, preview_magn: Any, normal_magn: Any, scl_magn: Any) -> None: ...
    def get(self): ...

class PanesRecord(BiffRecord):
    valid_active_pane: Any = ...
    def __init__(self, px: Any, py: Any, first_row_bottom: Any, first_col_right: Any, active_pane: Any) -> None: ...

class RowRecord(BiffRecord):
    def __init__(self, index: Any, first_col: Any, last_col: Any, height_options: Any, options: Any) -> None: ...

class LabelSSTRecord(BiffRecord):
    def __init__(self, row: Any, col: Any, xf_idx: Any, sst_idx: Any) -> None: ...

class MergedCellsRecord(BiffRecord):
    def __init__(self, merged_list: Any) -> None: ...
    def get(self): ...

class MulBlankRecord(BiffRecord):
    def __init__(self, row: Any, first_col: Any, last_col: Any, xf_index: Any) -> None: ...

class BlankRecord(BiffRecord):
    def __init__(self, row: Any, col: Any, xf_index: Any) -> None: ...

class RKRecord(BiffRecord):
    def __init__(self, row: Any, col: Any, xf_index: Any, rk_encoded: Any) -> None: ...

class NumberRecord(BiffRecord):
    def __init__(self, row: Any, col: Any, xf_index: Any, number: Any) -> None: ...

class BoolErrRecord(BiffRecord):
    def __init__(self, row: Any, col: Any, xf_index: Any, number: Any, is_error: Any) -> None: ...

class FormulaRecord(BiffRecord):
    def __init__(self, row: Any, col: Any, xf_index: Any, rpn: Any, calc_flags: int = ...) -> None: ...

class GutsRecord(BiffRecord):
    def __init__(self, row_gut_width: Any, col_gut_height: Any, row_visible_levels: Any, col_visible_levels: Any) -> None: ...

class WSBoolRecord(BiffRecord):
    def __init__(self, options: Any) -> None: ...

class ColInfoRecord(BiffRecord):
    def __init__(self, first_col: Any, last_col: Any, width: Any, xf_index: Any, options: Any, unused: Any) -> None: ...

class CalcModeRecord(BiffRecord):
    def __init__(self, calc_mode: Any) -> None: ...

class CalcCountRecord(BiffRecord):
    def __init__(self, calc_count: Any) -> None: ...

class RefModeRecord(BiffRecord):
    def __init__(self, ref_mode: Any) -> None: ...

class IterationRecord(BiffRecord):
    def __init__(self, iterations_on: Any) -> None: ...

class DeltaRecord(BiffRecord):
    def __init__(self, delta: Any) -> None: ...

class SaveRecalcRecord(BiffRecord):
    def __init__(self, recalc: Any) -> None: ...

class PrintHeadersRecord(BiffRecord):
    def __init__(self, print_headers: Any) -> None: ...

class PrintGridLinesRecord(BiffRecord):
    def __init__(self, print_grid: Any) -> None: ...

class GridSetRecord(BiffRecord):
    def __init__(self, print_grid_changed: Any) -> None: ...

class DefaultRowHeightRecord(BiffRecord):
    def __init__(self, options: Any, def_height: Any) -> None: ...

class DefColWidthRecord(BiffRecord):
    def __init__(self, def_width: Any) -> None: ...

class HorizontalPageBreaksRecord(BiffRecord):
    def __init__(self, breaks_list: Any) -> None: ...

class VerticalPageBreaksRecord(BiffRecord):
    def __init__(self, breaks_list: Any) -> None: ...

class HeaderRecord(BiffRecord):
    def __init__(self, header_str: Any) -> None: ...

class FooterRecord(BiffRecord):
    def __init__(self, footer_str: Any) -> None: ...

class HCenterRecord(BiffRecord):
    def __init__(self, is_horz_center: Any) -> None: ...

class VCenterRecord(BiffRecord):
    def __init__(self, is_vert_center: Any) -> None: ...

class LeftMarginRecord(BiffRecord):
    def __init__(self, margin: Any) -> None: ...

class RightMarginRecord(BiffRecord):
    def __init__(self, margin: Any) -> None: ...

class TopMarginRecord(BiffRecord):
    def __init__(self, margin: Any) -> None: ...

class BottomMarginRecord(BiffRecord):
    def __init__(self, margin: Any) -> None: ...

class SetupPageRecord(BiffRecord):
    def __init__(self, paper: Any, scaling: Any, start_num: Any, fit_width_to: Any, fit_height_to: Any, options: Any, hres: Any, vres: Any, header_margin: Any, footer_margin: Any, num_copies: Any) -> None: ...

class NameRecord(BiffRecord):
    def __init__(self, options: Any, keyboard_shortcut: Any, name: Any, sheet_index: Any, rpn: Any, menu_text: str = ..., desc_text: str = ..., help_text: str = ..., status_text: str = ...) -> None: ...

class ExternSheetRecord(BiffRecord):
    refs: Any = ...
    def __init__(self, refs: Any) -> None: ...
    def get(self): ...

class SupBookRecord(BiffRecord): ...

class InternalReferenceSupBookRecord(SupBookRecord):
    def __init__(self, num_sheets: Any) -> None: ...

class XcallSupBookRecord(SupBookRecord):
    def __init__(self) -> None: ...

class ExternnameRecord(BiffRecord):
    def __init__(self, options: int = ..., index: int = ..., name: Optional[Any] = ..., fmla: Optional[Any] = ...) -> None: ...
