# Stubs for xlwt.Cell (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class StrCell:
    rowx: Any = ...
    colx: Any = ...
    xf_idx: Any = ...
    sst_idx: Any = ...
    def __init__(self, rowx: Any, colx: Any, xf_idx: Any, sst_idx: Any) -> None: ...
    def get_biff_data(self): ...

class BlankCell:
    rowx: Any = ...
    colx: Any = ...
    xf_idx: Any = ...
    def __init__(self, rowx: Any, colx: Any, xf_idx: Any) -> None: ...
    def get_biff_data(self): ...

class MulBlankCell:
    rowx: Any = ...
    colx1: Any = ...
    colx2: Any = ...
    xf_idx: Any = ...
    def __init__(self, rowx: Any, colx1: Any, colx2: Any, xf_idx: Any) -> None: ...
    def get_biff_data(self): ...

class NumberCell:
    rowx: Any = ...
    colx: Any = ...
    xf_idx: Any = ...
    number: Any = ...
    def __init__(self, rowx: Any, colx: Any, xf_idx: Any, number: Any) -> None: ...
    def get_encoded_data(self): ...
    def get_biff_data(self): ...

class BooleanCell:
    rowx: Any = ...
    colx: Any = ...
    xf_idx: Any = ...
    number: Any = ...
    def __init__(self, rowx: Any, colx: Any, xf_idx: Any, number: Any) -> None: ...
    def get_biff_data(self): ...

error_code_map: Any

class ErrorCell:
    rowx: Any = ...
    colx: Any = ...
    xf_idx: Any = ...
    number: Any = ...
    def __init__(self, rowx: Any, colx: Any, xf_idx: Any, error_string_or_code: Any) -> None: ...
    def get_biff_data(self): ...

class FormulaCell:
    rowx: Any = ...
    colx: Any = ...
    xf_idx: Any = ...
    frmla: Any = ...
    calc_flags: Any = ...
    def __init__(self, rowx: Any, colx: Any, xf_idx: Any, frmla: Any, calc_flags: int = ...) -> None: ...
    def get_biff_data(self): ...
